proc loadDirectory {root mask0 target0 hist} {
    upvar $mask0 mask $target0 target
    set w $root.directories.listbox
    $w delete 0 end
    foreach i [lsort [glob -nocomplain [file dirname $mask]/{*,..}]] {
	if [file isdirectory $i] {$w insert end [file tail $i]}
    }
    set active [$w curselection]
    if {$active == ""} {set active 0}
    $w activate $active
    $w selection clear 0 end
    $w selection set active
    set w $root.files.listbox
    $w delete 0 end
    if [file isdirectory $mask] {set mask $mask/*}
    foreach i [lsort [glob -nocomplain $mask]] {
	if [file isfile      $i] {$w insert end [file tail $i]}
    }
    $w activate 0
    $w selection clear 0 end
    $w selection set active
    set target [file dirname $mask]/
    if $hist {$root.title.entry configure -text ""}
    if [$root.files.listbox index end] {
	focus $root.files.listbox
    } {
	focus $root.directories.listbox
    }
}

proc changeDirectory {root mask0 target0 hist} {
    upvar $mask0 mask $target0 target
    set w $root.directories.listbox
    focus $w
    set index [$w curselection]
    if {$index == ""} return
    set currentDirectory [pwd]
    if ![catch "cd [file dirname $mask]/[$w get $index]"]\
	    {set mask [pwd]/[file tail $mask]}
    cd $currentDirectory
    loadDirectory $root mask target $hist
}

proc loadTarget {root mask0 target0 hist} {
    upvar $mask0 mask $target0 target
    set w $root.files.listbox
    focus $w
    set index [$w curselection]
    if {$index == ""} return
    set target [file dirname $mask]/[$w get $index]
    if $hist {
	loadTitle $root mask target
    } {
	focus $root.target.entry
    }
}

proc loadTitle {root mask0 target0} {
    upvar $mask0 mask $target0 target
    if {[file dirname $target] == "."} {
	set target [file dirname $mask]/$target
    }
    if [file exists $target] {
	set fd [open "| [LibDir hpklist] $target" r]
	gets $fd title
	close $fd
	if {[set len [string length $title]] > 36} {
	    set title "[string range $title 0 24]...\
		       [string range $title [expr $len-11] end]"
	}
	$root.title.entry configure -text $title
#	if {$title != "<< NO Title >>"} {focus $root.buttons.ok}
    } else {
	tk_dialog .error "No File"\
		"File does NOT Exist.\n\"$target\"" error 0 OK
	$root.title.entry configure -text ""
    }
}

proc listTitle mask0 {
    upvar $mask0 mask
    if {[glob -nocomplain $mask] == ""} {
	tk_dialog .error "Error" "No File Matching" error 0 OK
	return
    }
    set W .titleList
    if ![winfo exists $W] {
	toplevel $W
	button $W.button -text Dismiss -underline 0 -command "wm withdraw $W"
	listbox $W.listbox -yscrollcommand "$W.scrollbar set"
	scrollbar $W.scrollbar -command "$W.listbox yview"
	pack $W.button -side bottom
	pack $W.scrollbar -side right -fill y
	pack $W.listbox -fill both
	bind $W <Alt-d> "$W.button invoke"
	wm withdraw $W
        update idletasks
        set x [expr [winfo screenwidth  $W]/2 - [winfo reqwidth  $W]/2 \
                - [winfo vrootx [winfo parent $W]]]
        set y [expr [winfo screenheight $W]/2 - [winfo reqheight $W]/2 \
                - [winfo vrooty [winfo parent $W]]]
        wm geom $W +$x+$y
    }
    wm title $W $mask
    wm iconname $W "Polly/TitleList"
    set w $W.listbox
    $w delete 0 end
    set namlen 0
    foreach i [lsort [glob -nocomplain $mask]] {
	if [file isfile $i] {
	    set l [string length [file tail $i]]
	    if {$l>$namlen} [set namlen $l]
	}
    }
    set form "%-${namlen}s    %s"
    set ttllen 0
    foreach i [lsort [glob -nocomplain $mask]] {
	if [file isfile      $i] {
	    set fd [open "| [LibDir hpklist] $i" r]
	    gets $fd title
	    close $fd
	    $w insert end [format $form [file tail $i] $title]
	    set l [string length $title]
	    if {$l>$ttllen} [set ttllen $l]
	}
    }
    set lines [$w index end]
    $w configure -height [expr {$lines>15} ? 15 : $lines]
    $w configure -width  [expr $namlen + $ttllen + 4]
    wm deiconify $W
}

proc genFileExit {root OK proc target0} {
    upvar $target0 target
    global ActiveEntry
    focus $ActiveEntry
    catch {wm withdraw .titleList}
    wm withdraw $root
    if {$OK && $proc != ""} {
	if [string match [set pwd [pwd]/]* $target]\
		{set target [string range $target [string length $pwd] end]}
	$proc $target
    }
}

proc genFile {root title mask target hist proc write} {
    global $mask $target

if ![winfo exists $root] {

    toplevel $root
    wm title    $root $title
    wm iconname $root $title

    set w $root.buttons
    frame $w
    button $w.ok -text OK\
	    -command "\
	    if {\$$target == \"\" || \[file isdirectory \$$target\]}\
		{loadTarget $root $mask $target $hist};\
	    genFileExit $root 1 \"$proc\" $target"
    button $w.cancel -text Cancel\
	    -command "genFileExit $root 0 {} {}"
    if $hist {
	button $w.titleList -text "Title List" -underline 6\
		-command "listTitle $mask"
	pack $w.ok $w.titleList $w.cancel -side left -padx 3m
	bind $root <Alt-l> "$w.titleList invoke"
    } {
	pack $w.ok $w.cancel -side left -padx 3m
    }
    bind $root <Escape> "$w.cancel invoke"
    pack $w -side bottom -pady 3m

    set w $root.target
    frame $w
    frame $w.space -height 2
    label $w.label -text "Target File:" -underline 0
    entry $w.entry -textvariable $target
    set font [$w.entry cget -font]
    pack  $w.space -side top
    pack  $w.label -side top -anchor w
    pack  $w.entry -side bottom -fill x -expand 1
    bind  $w.entry <Return> "$root.buttons.ok invoke"
    bind  $w.entry <Up>     "focus $root.files.listbox"
    bind  $w.entry <Down>   "focus $root.files.listbox"
    bind  $root <Alt-t> "focus $w.entry"
    pack  $w -side top -fill x -padx 2

    if $hist {
	set w $root.title
	frame $w
	label $w.label -text "Title:"
	label $w.entry -height 1 -font $font -justify left
	pack  $w.label -side top -anchor w
	pack  $w.entry -side bottom -anchor w
	pack  $w -side top -fill x -padx 2
    }

#   frame $root.space -height 3m
#   pack  $root.space -side top

    set w $root.mask
    frame $w
    label $w.label -text "Mask:" -underline 0
    entry $w.entry -textvariable $mask
    pack  $w.label -side top -anchor w
    pack  $w.entry -side bottom -fill x -expand 1
    bind  $w.entry <Return> "loadDirectory $root $mask $target $hist"
    bind  $root <Alt-m> "focus $w.entry"
    pack  $w -side bottom -fill x -padx 2

    set w $root.directories
    frame $w
    label $w.label -text "Directories:" -underline 0
    listbox $w.listbox -yscrollcommand "$w.scrollbar set"
    scrollbar $w.scrollbar -command "$w.listbox yview"
    pack $w.label -side top -anchor w
    pack $w.scrollbar -side right -fill y
    pack $w.listbox -fill both
    bind $w.listbox <FocusIn>  {%W selection set active}
    bind $w.listbox <Double-1> "changeDirectory $root $mask $target $hist"
    bind $w.listbox <Return>   "changeDirectory $root $mask $target $hist"
    bind $w.listbox <space>    "changeDirectory $root $mask $target $hist"
    bind $w.listbox <Right>    "focus $root.files.listbox"
    bind $w.listbox <Control-f> "focus $root.files.listbox"
    bind $w.listbox <KeyPress> {
	if [regexp {[a-zA-Z]} %A] {
	    for {set i 0} {$i < [%W index end]} {incr i} {
		if [string match "%A*" [%W get $i]] {
		    %W activate $i
		    %W selection clear 0 end
		    %W selection set $i
		    %W see $i
		    break
		}
	    }
	}
    }
    bind $root <Alt-d> "focus $w.listbox"
    pack $w -side left -fill y -padx 2

    set w $root.files
    frame $w
    label $w.label -text "Files:" -underline 0
    listbox $w.listbox -yscrollcommand "$w.scrollbar set"
    scrollbar $w.scrollbar -command "$w.listbox yview"
    pack $w.label -side top -anchor w
    pack $w.scrollbar -side right -fill y
    pack $w.listbox -fill both
    bind $w.listbox <FocusIn>  {%W selection set active}
    bind $w.listbox <ButtonRelease-1> "loadTarget $root $mask $target $hist"
    bind $w.listbox <space>    "loadTarget $root $mask $target $hist"
    bind $w.listbox <Double-1> "loadTarget $root $mask $target 0
				genFileExit $root 1 \"$proc\" $target"
    bind $w.listbox <Return>   "loadTarget $root $mask $target 0
				genFileExit $root 1 \"$proc\" $target"
    bind $w.listbox <Left>     "focus $root.directories.listbox"
    bind $w.listbox <Control-b> "focus $root.directories.listbox"
    bind $w.listbox <KeyPress> {
	if [regexp {[a-zA-Z]} %A] {
	    for {set i 0} {$i < [%W index end]} {incr i} {
		if [string match "%A*" [%W get $i]] {
		    %W activate $i
		    %W selection clear 0 end
		    %W selection set $i
		    %W see $i
		    break
		}
	    }
	}
    }
    bind $root <Alt-f> "focus $w.listbox"
    pack $w -fill both -padx 2

    wm withdraw $root
    update idletasks
    set x [expr [winfo screenwidth  $root]/2 - [winfo reqwidth  $root]/2 \
	    - [winfo vrootx [winfo parent $root]]]
    set y [expr [winfo screenheight $root]/2 - [winfo reqheight $root]/2 \
	    - [winfo vrooty [winfo parent $root]]]
    wm geom $root +$x+$y
}

set currentDirectory [pwd]
set cmd "cd \[file dirname \$$mask\]; set $mask \[pwd\]/\[file tail \$$mask\]"
catch "eval $cmd"
cd $currentDirectory
loadDirectory $root $mask $target $hist
$root.mask.entry icursor end
$root.mask.entry xview insert
$root.target.entry icursor end
$root.target.entry xview insert
if $write {focus $root.target.entry}

wm deiconify $root

}

#--------------------------------------------------------

proc includeSelect target {
    global histList

    set w .includeList
    set h $w.hist
    set b $w.buttons
    set s $w.select

    if ![winfo exists $w] {
	toplevel    $w
	wm iconname $w "Polly/Include"

	frame $h -relief groove -bd 3
	pack  $h -side top -fill x -expand 1 -padx 1m -pady 1m
	label $h.label -text "Histogram List" -underline 10
	listbox $h.listbox -selectmode extended -setgrid 1\
		-yscrollcommand "$h.scrollbar set"
	scrollbar $h.scrollbar -command "$h.listbox yview"
	pack $h.label -side top
	pack $h.scrollbar -side right -fill y -pady 1m
	pack $h.listbox -expand 1 -fill both -pady 1m
	bind $w <Alt-l> "focus $h.listbox"

	frame $b
	pack  $b -side top
	frame  $b.select 
	button $b.select.image -image imageDOWN -command {
	    set h .includeList.hist.listbox
	    set s .includeList.select.listbox
	    foreach i [$h curselection] {
		scan [$h get $i] "%d:" id
		for {set j 0} {$j < [$s size]} {incr j}\
			{scan [$s get $j] "%d:" jd; if {$jd > $id} break}
		$s insert $j [$h get $i]
	    }
	    foreach i [lsort -decreasing [$h curselection]] {$h delete $i}
	}
	label  $b.select.label -text Select -underline 0
	pack   $b.select.image $b.select.label -side left
	frame  $b.unselect
	button $b.unselect.image -image imageUP -command {
	    set h .includeList.hist.listbox
	    set s .includeList.select.listbox
	    foreach i [$s curselection] {
		scan [$s get $i] "%d:" id
		for {set j 0} {$j < [$h size]} {incr j}\
			{scan [$h get $j] "%d:" jd; if {$jd > $id} break}
		$h insert $j [$s get $i]
	    }
	    foreach i [lsort -decreasing [$s curselection]] {$s delete $i}
	}
	label  $b.unselect.label -text Unselect -underline 0
	pack   $b.unselect.image $b.unselect.label -side left
	menubutton $b.include -text Include -underline 0 -menu $b.include.m\
		-padx 3 -pady 2 -relief raised -bd 2 -highlightthickness 2
	menu  $b.include.m -tearoff 0\
		-postcommand {.hist.listbox selection set active}
	$b.include.m add command -label "Before active line" -underline 0
	$b.include.m add command -label "After active line" -underline 0
	button $b.cancel -text Cancel -padx 3 -pady 2
	pack   $b.select $b.unselect $b.include $b.cancel -side left -padx 1m
	bind   $w <Alt-s>  "$b.select.image   invoke"
	bind   $w <Alt-u>  "$b.unselect.image invoke"
	bind   $w <Alt-i>  "tkMbPost $b.include"
	bind   $w <Return> "tkMbPost $b.include"
	bind   $w <Escape> "$b.cancel invoke"

	frame $s -relief groove -bd 3
	pack  $s -side bottom -fill x -expand 1 -padx 1m -pady 1m
	label $s.label -text "Selected Histogram" -underline 9
	listbox $s.listbox -selectmode extended -setgrid 1\
		-yscrollcommand "$s.scrollbar set"
	scrollbar $s.scrollbar -command "$s.listbox yview"
	pack $s.label -side top
	pack $s.scrollbar -side right -fill y -pady 1m
	pack $s.listbox -expand 1 -fill both -pady 1m
	bind $w <Alt-h> "focus $s.listbox"

	bind $h.listbox <Double-1> "$b.select.image   invoke"
	bind $h.listbox <space>    "$b.select.image   invoke"
	bind $s.listbox <Double-1> "$b.unselect.image invoke"
	bind $s.listbox <space>    "$b.unselect.image invoke"

	wm withdraw $w
	update idletasks
	set x [expr [winfo screenwidth  $w]/2 - [winfo reqwidth  $w]/2 \
		- [winfo vrootx [winfo parent $w]]]
	set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
		- [winfo vrooty [winfo parent $w]]]
	wm geom $w +$x+$y
    }
    if [file exists $target] {
	set fd [open "| [LibDir hpklist] -l $target" r]
	gets $fd title
	$h.listbox delete 0 end
	while {[gets $fd line] != -1} {$h.listbox insert end $line}
	close $fd
	$h.listbox selection clear 0 end
	$h.listbox activate 0
	$h.listbox selection set active
    } else {
	tk_dialog .error "No File"\
		"File does NOT Exist.\n\"$target\"" error 0 OK
	set title ""
    }
    $s.listbox delete 0 end
    $h.label configure -text "Histogram List \"$title\""
    foreach i {0 1} {
	$b.include.m entryconfigure $i -command\
		"focus [focus]; wm withdraw $w;\
                 includeAction $s.listbox $target $i"
    }
    proc includeAction {w target flag} {
	for {set i 0} {$i < [$w size]} {incr i} {
	    regexp {^ *([0-9]+):} [$w get $i] dummy id
	    lappend IDlist $id
	}
        set insert [expr [.hist.listbox size] ?\
                [expr [.hist.listbox index active]+$flag+1] : 1]
        H File Include $target $insert [join $IDlist]
    }    
    $b.cancel  configure\
	    -command "focus [focus]; wm withdraw $w"
    wm title $w "Polly/Include \"[file tail $target]\""
    wm deiconify $w
    focus $h.listbox
}

#--------------------------------------------------------

proc FileCumulate target {
    if [file exists $target] {
	H File Cumulate $target
    } else {
	tk_dialog .error "No File" \
		"File does NOT Exist.\n\"$target\"" error 0 OK
    }
}

#--------------------------------------------------------

proc FileSaveAction target {
    if [file exists $target] {
	set arg $target
	while {[file type $arg] == "link"} {set arg [file readlink $arg]}
	if [file isfile $arg] {
	    if {[Confirm "Overwrite\n\"[file tail $target]\" ?" ok]\
		    != "ok"} return
	} {
	    tk_dialog .error "Illigal File name"\
		    "Illigal File Name\n\"$target\"" error 0 OK
	    return
	}
    }
    H File Save $target
}

#--------------------------------------------------------

proc MacroWriteAction target {
    global macroSelection
    if [file exists $target] {
	set arg $target
	while {[file type $arg] == "link"} {set arg [file readlink $arg]}
	if [file isfile $arg] {
	    if {[Confirm "Overwrite\n\"[file tail $target]\" ?" ok]\
		    != "ok"} return
	} {
	    tk_dialog .error "Illigal File name"\
		    "Illigal File Name\n\"$target\"" error 0 OK
	    return
	}
    }
    if [catch {set fd [open $target w]}] {
	tk_dialog .error "File Open Error"\
		"File Open Error\n\"$target\"" error OK
	return
    }
    puts $fd $macroSelection
    close $fd
    putLog "# Macro Written -> \"$target\"\n" "tag color1"
}

#--------------------------------------------------------

proc CursorReadAction target {
    if [file exists $target] {
	H Cursor Read $target
    } else {
	tk_dialog .error "No File" \
		"File does NOT Exist.\n\"$target\"" error 0 OK
    }
}

#--------------------------------------------------------

proc histFileOpen {} {
    genFile .openFile "Polly/FileOpen" histMask histTarget 1 readHPK 0
}

proc histFileInclude {} {
    global includeMask histMask
    if ![info exists includeMask] {set includeMask $histMask}
    genFile .includeFile "Polly/FileInclude"\
	    includeMask includeTarget 1 includeSelect 0
}

proc histFileCumulate {} {
    genFile .cumulFile "Polly/FileCumulate"\
	    histMask histTarget 1 FileCumulate 0
}

proc histFileSave {} {
    genFile .saveFile "Polly/FileSave" histMask histTarget 0 FileSaveAction 1
}

proc macroFileRead {} {
    genFile .readMacro "Polly/MacroRead" macroMask macroTarget 0 M 0
}

proc macroFileWrite {} {
    global macroSelection
    set w .macro.text
    if {[$w tag ranges sel] == ""} {
	tk_dialog .error "Empty Selection"\
		"Empty Selected Area"  error 0 OK
	return
    }
    # save selection because it will disappear in the next menu...
    set macroSelection [$w get [$w index sel.first] [$w index sel.last]]
    genFile .writeMacro "Polly/MacroWrite"\
	    macroMask macroTarget 0 MacroWriteAction 1
}

proc cursorFileRead {} {
    genFile .readCursor "Polly/CursorRead"\
	    cursorMask cursorTarget 0 CursorReadAction 0
}
