set root .menu.help.m

#-------------------------------------------------

source [LibDir html.tcl]

#-------------------------------------------------

proc helpPalette {} {
    global ColorPaletteR ColorPaletteG ColorPaletteB ColorPaletteDefault

    set w .helpPalette
    if ![winfo exists $w] {
	toplevel $w
	wm title    $w "Polly/ColorPalette"
	wm iconname $w "Polly/ColorPalette"
	button $w.new     -text "New\nBackground\nColor"     -relief raised
	button $w.default -text "Default\nBackground\nColor" -relief raised
	scale $w.scaleR -label Red: -variable ColorPaletteR\
		-orient horizontal -from 0 -to 255 -command showPalette
	scale $w.scaleG -label Green: -variable ColorPaletteG\
		-orient horizontal -from 0 -to 255 -command showPalette
	scale $w.scaleB -label Blue: -variable ColorPaletteB\
		-orient horizontal -from 0 -to 255 -command showPalette
	proc showPalette dummy {
	    global ColorPaletteR ColorPaletteG ColorPaletteB
	    .helpPalette.new configure -background\
		    [format "#%2.2X%2.2X%2.2X"\
		    $ColorPaletteR $ColorPaletteG $ColorPaletteB]
	}
	frame $w.buttons
	pack $w.buttons -side bottom -pady 2m
	pack $w.scaleB $w.scaleG $w.scaleR -side bottom -fill x -expand 1
	pack $w.new     -padx 3m -pady 3m -fill y    -side left
	pack $w.default -padx 3m -pady 3m -fill both
	set b $w.buttons
	button $b.apply   -text Apply   -underline 0 -command "\
	    tk_setPalette \[format \"#%2.2X%2.2X%2.2X\"\
		    \$ColorPaletteR \$ColorPaletteG \$ColorPaletteB\];\
	    $w.default configure -background \$ColorPaletteDefault"
	button $b.default -text Default -underline 0 -command "
	    tk_setPalette \$ColorPaletteDefault;\
	    $w.new configure -background \[format \"#%2.2X%2.2X%2.2X\"\
		    \$ColorPaletteR \$ColorPaletteG \$ColorPaletteB\]"
	button $b.cancel  -text Cancel
	pack $b.apply $b.default $b.cancel -side left -padx 2m
	bind $w <Alt-a>  "$b.apply   invoke"
	bind $w <Alt-d>  "$b.default invoke"
	bind $w <Escape> "$b.cancel  invoke"
	set ColorPaletteR 192
	set ColorPaletteG 192
	set ColorPaletteB 192
	set ColorPaletteDefault [lindex [. configure -background] 3]
	wm withdraw $w
	update idletasks
	set x [expr [winfo screenwidth  $w]/2 - [winfo reqwidth  $w]/2 \
		- [winfo vrootx [winfo parent $w]]]
	set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
		- [winfo vrooty [winfo parent $w]]]
	wm geom $w +$x+$y
    }
    $w.buttons.cancel configure -command "focus [focus]; wm withdraw $w"
    wm deiconify $w
}

#-------------------------------------------------

proc helpXres {} {

    set maxdepth 9
    set root .helpXres
    set w $root.widgets
    set l $w.listbox
    set s $w.scrollbar
    set o $root.options
    set b $root.buttons

    if ![winfo exists $root] {
	toplevel $root
	wm title    $root "Polly/X-ResEdit"
	wm iconname $root "Polly/X-ResEdit"

	frame $w
	frame $o
	frame $b
	pack  $w $o -fill x -padx 2m -pady 2m
	pack  $b -pady 3m

	for {set i 0} {$i <= $maxdepth} {incr i} {
	    listbox $l$i -yscrollcommand "$s$i set"
	    scrollbar $s$i -command "$l$i yview"
	    bind $l$i <Double-1> "listWtree W $l $s $i $maxdepth $o"
	    bind $l$i <Return>   "listWtree W $l $s $i $maxdepth $o"
	}

	label $o.widget -text Widget:
	label $o.class  -text Class:
	label $o.options -text Options:
	text  $o.text  -state disabled -wrap word -width 40\
		-spacing1 3 -spacing2 3 -cursor hand1
	label $o.name   -text Resource:
	label $o.value  -text Value:
	label $o.default -text Default:
	frame $o.changeto
	pack  $o.widget $o.class $o.options -anchor w ;# -pady 1m
	pack  $o.text  -fill x -expand 1
	pack  $o.name $o.value $o.default -anchor w
	pack  $o.changeto -fill x
	$o.text tag configure option
	$o.text tag configure hot -background green
	$o.text tag bind option <Enter> "\
		set word1 \[$o.text index {@\%x,\%y wordstart}\];\
		$o.text tag add hot \"\$word1 -1c\" \"\$word1 wordend\""
	$o.text tag bind option <Motion> "\
		set word2 \[$o.text index {@\%x,\%y wordstart}\];\
		if {\[string compare \$word1 \$word2\] != 0} {
		    set word1 \$word2
		    $o.text tag add hot \"\$word1 -1c\" \"\$word1 wordend\"}"
	$o.text tag bind hot <Button-1> "\
		if !\[info exists W\] break;\
		set word   \[$o.text index {@\%x,\%y wordstart}\];\
		set option \[$o.text get \$word \"\$word wordend\"\];\
		$o.changeto.entry delete 0 end
		set list   \[\$W configure -\$option\];\
		$o.name  configure -text \"Resource:\\t\[lindex \$list 1\]\";\
		$o.value configure -text \"Value:\\t\\t\[lindex \$list 4\]\";\
		$o.default configure\
		                 -text \"Default:\\t\\t\[lindex \$list 3\]\""
	$o.text tag bind hot <Leave> "$o.text tag remove hot 1.0 end"

	set c  $o.changeto
	label  $c.label -text "Change to:\t"
	entry  $c.entry
	pack $c.label -side left -anchor w
	pack $c.entry -fill x -expand 1

	button $b.apply -text "Apply" -command "\
		set newval \[$c.entry get\];\
		if {\$newval != \"\"}\
		    {uplevel #0 \"\$W configure -\$option \$newval\"}"
	button $b.dismiss -text "Dismiss"
	pack $b.apply $b.dismiss -side left -padx 4m
	bind $c.entry <Return> "$b.apply invoke"
	bind $root    <Escape> "$b.dismiss invoke"

	wm withdraw $root
	update idletasks
	set x [expr [winfo screenwidth  $root]/2 - [winfo reqwidth  $root]/2 \
		- [winfo vrootx [winfo parent $root]]]
	set y [expr [winfo screenheight $root]/2 - [winfo reqheight $root]/2 \
		- [winfo vrooty [winfo parent $root]]]
	wm geom $root +$x+$y
    }

    $b.dismiss configure -command "focus [focus]; wm withdraw $root"

    set W .
    listWtree W $l $s -1 $maxdepth $o

    wm deiconify $root
}

proc listWtree {W0 l s d n o} {
    global WidgetOptions
    upvar $W0 W

    if {$d >= $n} return
    for {set i [expr $d+1]} {$i <= $n} {incr i} {$l$i delete 0 end}
    if {$d >= 0} {
	if [regexp -nocase {\[[a-z0-9]*\]} [set w [$l$d get active]]] return
	for {set i $d} {$i >= 0} {incr i -1} {set w [$l$i get 0].$w}
	regsub -nocase -all {(^\[[a-z0-9 #]*\]|\[|\])} $w "" W
    } {
	set W .
    }
    $o.widget configure -text "Widget:\t$W"
    $o.class  configure -text "Class:\t[set Class [winfo class $W]]"
    if ![info exists WidgetOptions($Class)] {
	foreach i [$W configure] {lappend WidgetOptions($Class) [lindex $i 0]}
    }
    $o.text configure -state normal
    $o.text delete 0.0 end
    foreach i $WidgetOptions($Class) {
	$o.text insert end - {} [string range $i 1 end] option " "
    }
    $o.text    configure -state disabled
    $o.name    configure -text Resource:
    $o.value   configure -text Value:
    $o.default configure -text Default:

    if {[set m [winfo children $W]] == {}} return
    $l[incr d] insert end "\[[winfo name $W]\]"
    foreach i $m {$l$d insert end [winfo name $i]}
    packWtree $l $s $d $n
    $l$d activate 1
    $l$d selection set active
#   focus $l$d
}

proc packWtree {l s d n} {
    if {$d >= $n} {return $n}
    set nw 2
    for {set i 0} {$i <= $n} {incr i} {catch {pack forget $l$i $s$i}}
    for {
	set i [expr {$d<$nw?0:[expr $d-$nw]}]
	set j [expr $i+$nw]
    } {$i <= $j} {incr i} {
	pack $l$i -side left -fill y -expand 1
	pack $s$i -side left -fill y -expand 1
    }
}

#-------------------------------------------------

proc helpHPK {} {
    global TextSearchTagForeground TextSearchTagBackground

    set w .helpHPK
    set t $w.text
    set b $w.buttons

    if ![winfo exists $w] {
	set lpage 60
	toplevel $w
	wm withdraw $w
	wm title    $w "HandyPak"
	wm iconname $w "HandyPak"
	text $t -width 67 -height [expr $lpage+2]\
		-relief sunken -setgrid 1 -yscrollcommand "$w.scrollbar set"
	scrollbar $w.scrollbar -command "$t yview"
	frame $w.buttons
	pack $b -side bottom -pady 2m
	pack $w.scrollbar -side right -fill y -pady 1m
	pack $t -fill both -expand 1 -padx 1m -pady 1m
	$t tag configure search\
		-foreground $TextSearchTagForeground\
		-background $TextSearchTagBackground

	button $b.align  -text ALign  -underline 1 -command "helpAlign $t  0"
	bind $t <Alt-l> "$b.align  invoke"
	bind $t <Control-l> "$b.align  invoke"
	button $b.next   -text NextPg -underline 0 -command "helpAlign $t +1"
	bind $t <Alt-n> "$b.next   invoke"
	bind $t <Alt-f> "$b.next   invoke"
	button $b.prev   -text PrevPg -underline 0 -command "helpAlign $t -1"
	bind $t <Alt-p> "$b.prev   invoke"
	bind $t <Alt-b> "$b.prev   invoke"
	button $b.search -text Search -underline 0 -command "helpSearch $t 1"
	bind $t <Alt-s> "$b.search invoke"
	button $b.dismiss -text "Dismiss" -underline 0
	bind $t <Alt-d> "$b.dismiss invoke"
	bind $t <Escape> "$b.dismiss invoke"
	pack $b.align $b.next $b.prev $b.search $b.dismiss\
		-side left -padx 2m
	bind $t <Control-s> "helpSearch $t  1"
	bind $t <Control-r> "helpSearch $t -1"

	update idletasks
	set x [expr [winfo screenwidth  $w]/2 - [winfo reqwidth  $w]/2 \
		- [winfo vrootx [winfo parent $w]]]
	set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
		- [winfo vrooty [winfo parent $w]]]
	wm geom $w +$x+$y
	wm deiconify $w

	if [catch {set fd [open [LibDir doc]/handypak r]}] {
	    tk_dialog .error "No File"\
		    "Manual \"handypak\"\nNOT Found." error 0 OK
	    catch {destroy $w}
	    return
	}
	set n 0
	while {[gets $fd line] != -1} {
	    if [regexp {^1} $line] {
		while {$n < $lpage} {
		    $t insert end "\n"
		    incr n
		}
		set n 0
	    }
	    $t insert end "$line\n"
	    incr n
	}
	close $fd
	$t mark set insert 0.0
	$t configure -state disabled
    }

    $b.dismiss configure -command\
	    "catch {wm withdraw .genSearch}; focus [focus]; wm withdraw $w"
    if {[wm state $w] != "normal"} {wm deiconify $w}

    focus $t
}

proc helpAlign {w dir} {
    set cur [$w index @0,0]
    switch -- $dir {
	+1 {
	    if {[set point [$w search -forward -regexp {^1} "$cur+1l" end]]\
		    == ""} {set point end}
	}
	 0 {
	    if {[set point [$w search -forward -regexp {^1} $cur end]]\
		    == ""} {set point $cur}
	}
	-1 {
	    if {[set point [$w search -backward -regexp {^1} "$cur-1l" 0.0]]\
		    == ""} {set point 0.0}
	}
    }
    $w yview $point
    set ins [$w index insert]
    if {[$w compare $ins <  $point] ||\
	[$w compare $ins > "$point+[expr [$w cget -height]-3]l"]}\
        {$w mark set insert $point}
}

#-------------------------------------------------

proc helpAbout {} {
    set oldFocus [focus]
    set w .helpAbout
    catch {destroy $w}
    toplevel $w
    wm title    $w "about Polly"
    wm iconname $w "about Polly"
    wm transient $w [winfo toplevel [winfo parent $w]]
    frame $w.space1 -height 2m
    label $w.symbol -image imagePOLLY
    label $w.label -text \
"An interactive display program
for the rawdata analysis
of Nuclear Physics experiments.\n
on VMS: 1989-1995
on UNIX: 1995-\n
H. Okamura
Department of Physics
Univ. of Tokyo
okamura@timshel.riken.go.jp\n
for info. and archive ...
http://iris.riken.go.jp/iris/polly/
"
    button $w.dismiss -text Dismiss -command "focus $oldFocus; destroy $w"
    frame $w.space2 -height 2m
    pack $w.space1 $w.symbol $w.label $w.dismiss $w.space2 -padx 3m -pady 1m
    bind $w <Return> "$w.dismiss invoke"
    bind $w <Escape> "$w.dismiss invoke"
    wm withdraw $w
    update idletasks
    set x [expr [winfo screenwidth  $w]/2 - [winfo reqwidth  $w]/2 \
	    - [winfo vrootx [winfo parent $w]]]
    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
	    - [winfo vrooty [winfo parent $w]]]
    wm geom $w +$x+$y
    wm deiconify $w
    focus $w.dismiss
}
