set searchCaseSense 0
set searchDirection 1
set searchFailedStr ""

proc genSearch {title tag procSearch procTag procExit} {
    global searchString searchCaseSense searchDirection

    set w .genSearch
    set o $w.options
    set b $w.buttons
    set e $w.entry

    if ![winfo exists $w] {
	toplevel    $w

	label $w.label -text "Search String:"
	entry $w.entry -textvariable searchString
	frame $w.options
	frame $w.buttons
	pack $w.buttons -side bottom -pady 3 -fill x
	pack $w.options -side bottom
	pack $w.label -side top -padx 4 -pady 2 -anchor w
	pack $w.entry -fill x -padx 4

	checkbutton $o.case -text "CaseSense" -underline 0\
		-variable searchCaseSense
	radiobutton $o.forward -text "Forward" -underline 0\
		-variable searchDirection -value 1
	radiobutton $o.backward -text "Backward" -underline 0\
		-variable searchDirection -value -1
	pack $o.case $o.forward $o.backward -side left

	button $b.search -text "Search"
	button $b.tag -text "Tag" -underline 0
	button $b.cancel -text "Cancel"
	pack $b.search $b.tag $b.cancel -side left -expand 1

	bind $e <Return>    "$b.search   invoke"
	bind $e <Alt-t>     "$b.tag      invoke"
	bind $e <Escape>    "$b.cancel   invoke"
	bind $e <Alt-c>     "$o.case     invoke"
	bind $e <Alt-f>     "$o.forward  invoke"
	bind $e <Alt-b>     "$o.backward invoke"
	bind $e <Control-s> "$o.forward  invoke; $b.search  invoke"
	bind $e <Control-r> "$o.backward invoke; $b.search  invoke"

	wm withdraw $w
	update idletasks
	set x [expr [winfo screenwidth  $w]/2 - [winfo reqwidth  $w]/2 \
		- [winfo vrootx [winfo parent $w]]]
	set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
		- [winfo vrooty [winfo parent $w]]]
	wm geom $w +$x+$y
    }

    set searchString ""
    wm title    $w $title
    wm iconname $w $title
    $b.search configure -command  $procSearch
    $b.tag    configure -command  $procTag -state $tag
    $b.cancel configure -command "$procExit; set searchFailedStr \"\";\
				  focus [focus]; wm withdraw $w"
    wm deiconify $w

    focus $e
}

#----------------------------------------------------------

proc histSearch {} {
    genSearch "Polly/HistSearch" disabled histSearchAction {} {}
}
proc histSearchAction {} {
    global searchString searchCaseSense searchDirection searchFailedStr
    if {$searchString == ""} return
    if $searchCaseSense\
	    {set arg $searchString}\
	    {set arg [string toupper $searchString]}
    set w .hist.listbox
    if {"$searchString" != "$searchFailedStr"} {
	set i [expr [$w index active]+ $searchDirection]
    } {
	set i [expr {$searchDirection==1 ? 0 : [expr [$w index end]-1]}]
    }
    for {} {$i >= 0 && $i < [$w index end]} {incr i $searchDirection} {
	regsub {^[ 0-9]+:} [$w get $i] "" line
	if !$searchCaseSense {set line [string toupper $line]}
	if {[string first $arg $line] != -1} {
	    histListboxSelect $i ""
	    set searchFailedStr ""
	    return
	}
    }
    tk_dialog .error "String Not Found"\
	    "String \"$searchString\" NOT Found" error 0 OK
    set searchFailedStr $searchString
}

#----------------------------------------------------------

proc macroSearch dir {
    global searchDirection
    set searchDirection $dir
    set w .macro.text
    genSearch "Polly/MacroSearch" normal\
	    "textSearchAction $w" "textTagAction $w"\
	    "$w tag remove search 0.0 end"
}

proc helpSearch {w dir} {
    global searchDirection
    set searchDirection $dir
    genSearch "Polly/HelpSearch" normal\
	    "textSearchAction $w" "textTagAction $w"\
	    "$w tag remove search 0.0 end"
}

proc textSearchAction w {
    global searchString searchCaseSense searchDirection searchFailedStr
    if {$searchString == ""} return
    set start [$w index insert]
    if {$searchDirection == 1} {
	if {"$searchString" == "$searchFailedStr"} {set start 1.0}
	if $searchCaseSense {
	    set cur [$w search -forwards\
		    -count length $searchString $start end]
	} {
	    set cur [$w search -forwards -nocase\
		    -count length $searchString $start end]
	}
    } {
	if {"$searchString" == "$searchFailedStr"} {set start [$w index end]}
	if $searchCaseSense {
	    set cur [$w search -backwards\
		    -count length $searchString $start 1.0]
	} {
	    set cur [$w search -backwards -nocase\
		    -count length $searchString $start 1.0]
	}
    }
    if {$cur == ""} {
	tk_dialog .error "String Not Found"\
		"String \"$searchString\" NOT Found" error 0 OK
	set searchFailedStr $searchString
    } {
	$w tag remove search 0.0 end
	$w tag add search $cur "$cur + $length char"
	if {$searchDirection == 1} {
	    tkTextSetCursor $w "$cur + $length char"
	} {
	    tkTextSetCursor $w $cur
	}
 	set searchFailedStr ""
    }
}
proc textTagAction w {
    global searchString searchCaseSense
    $w tag remove search 0.0 end
    if {$searchString == ""} return
    set cur 1.0
    while 1 {
	if $searchCaseSense {
	    set cur [$w search -count length $searchString $cur end]
	} {
	    set cur [$w search -nocase -count length $searchString $cur end]
	}
	if {$cur == ""} break
	$w tag add search $cur "$cur + $length char"
	set cur [$w index "$cur + $length char"]
    }
}

#----------------------------------------------------------

proc histSelectAction arg {
    global histList
    regsub -all {(-|\(|\)|/[A-Z,0-9.+-]+)} [string toupper $arg] " " arg
    regsub -all { +} [string trim $arg] " " arg
    set w .hist.listbox
    $w selection clear 0 end
    foreach key [split $arg " "] {
	for {set i 0} {$i < [llength $histList]} {incr i} {
	    if [string match $key [lindex $histList $i]] {$w selection set $i}
	}
    }
}
